@@ -1,7 +1,7 @@
 //! General purpose functionalities for computer science. Got any wishes? Tell us on GitHub or our Discord.
-use crate::math::general::Increment;
-use std::{intrinsics::transmute, fmt::Debug, ops::{BitAndAssign, Shl, BitOrAssign, BitXorAssign}, convert::TryFrom};
-use core::mem::{size_of_val, size_of};
+use crate::math::general::NumTools;
+use std::{intrinsics::transmute, fmt::Debug, ops::{*}, convert::TryFrom};
+use core::mem::*;
 
 const BITWISE_ERR1: &str = "Arguments were not the same size in memory.";
 const BITWISE_ERR2: &str = "U is bigger than T. Consider reversing the arguments.";
@@ -14,6 +14,53 @@ const EXPONENT_MASK_32:      u32 = 0b01111111100000000000000000000000;
 const SET_EXP_ZERO_MASK_32:  u32 = 0b00111111100000000000000000000000;
 const GET_SMANTISSA_MASK_32: u32 = 0b10000000011111111111111111111111;
 
+/// Trait for several functions that extend the functionality of slices.
+pub trait SliceOps<T> {
+    /// Look at the top of a slice `[T]`. Returns the top value but does not pop it.
+    /// # Returns
+    /// A `T`.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::compsci::general::SliceOps;
+    /// 
+    /// let v = vec![0.2, 3.2];
+    /// assert_eq!(3.2, v.peek());
+    /// assert_eq!(vec![0.2, 3.2], v);
+    #[must_use]
+    fn peek(&self) -> T;
+    /// Swap the top two elements of a slice `[T]`.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::compsci::general::SliceOps;
+    /// 
+    /// let mut v = vec![0.2, 3.2];
+    /// v.swap_top();
+    /// assert_eq!(vec![3.2, 0.2], v);
+    fn swap_top(&mut self);
+    /// Find the number of times a item is contained in a slice `[T]`.
+    /// # Returns
+    /// A `usize`.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::compsci::general::SliceOps;
+    /// 
+    /// let mut v = vec![4, 0, 0, 2, 3, 4, 4, 4];
+    /// assert_eq!(4, v.count_of(4));
+    #[must_use]
+    fn count_of(&self, item: T) -> usize;
+    /// Get the indexes at which a item resides in a slice `[T]`.
+    /// # Returns
+    /// A `Vec<usize>`.
+    /// # Examples
+    /// ```
+    /// use lib_rapid::compsci::general::SliceOps;
+    /// 
+    /// let mut v = vec![4, 0, 0, 2, 3, 4, 4, 4];
+    /// assert_eq!(vec![0, 5, 6, 7], v.positions_of(4));
+    #[must_use]
+    fn positions_of(&self, item: T) -> Vec<usize>;
+}
+
 /// Bitwise operations on slices of arbitrary (numeric) types.
 pub trait BitwiseSlice<T, U> {
     /// XORs a slice of type `[T]` with a slice of type `[U]`.
@@ -80,13 +127,10 @@ pub trait BitwiseSlice<T, U> {
 /// Trait for `binary_insert`.
 pub trait BinaryInsert<T> {
     /// Insert an element into a ***sorted*** `vec` whilst maintaining the order, consuming `other`.
-    ///
     /// # Arguments
     /// * `value` - The value which needs to be inserted.
-    ///
     /// # Returns
     /// Nothing.
-    ///
     /// # Warning
     /// This function ***will not*** check if the parsed `vec` is sorted.
     /// # Examples
@@ -127,108 +171,6 @@ pub trait BinaryInsert<T> {
     /// ```
     fn binary_insert_no_dup(&mut self, value: T);
 }
-/// Trait for a kind of indexing Strings in Rust.
-pub trait StringIndex {
-    /// Get the `char` at a given index from a `String` or `&str`.
-    /// If only dealing with ASCII-characters, `byte_at` is recommended.
-    /// 
-    /// # Arguments
-    /// * `index` - The index of the character
-    ///
-    /// # Returns
-    /// A `Option<char>`.
-    /// 
-    /// # Examples
-    /// ```
-    /// use lib_rapid::compsci::general::StringIndex;
-    /// 
-    /// let s = String::from("Hello");
-    /// assert_eq!('H', s.char_at(0).unwrap());
-    /// assert_eq!('e', s.char_at(1).unwrap());
-    /// assert_eq!('l', s.char_at(2).unwrap());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::StringIndex;
-    /// 
-    /// let s = "Hello";
-    /// assert_eq!('H', s.char_at(0).unwrap());
-    /// assert_eq!('e', s.char_at(1).unwrap());
-    /// assert_eq!('l', s.char_at(2).unwrap());
-    /// ```
-    #[must_use]
-    fn char_at(&self, index: usize) -> Option<char>;
-    /// Get the byte at a given index from a `String` or `&str`.
-    /// If dealing with Unicode characters, this function is *not* recommended, as it only returns one byte even though Unicode can be a multi byte encoding.
-    /// 
-    /// # Arguments
-    /// * `index` - The index of the byte
-    ///
-    /// # Returns
-    /// A `u8`.
-    /// 
-    /// # Examples
-    /// ```
-    /// use lib_rapid::compsci::general::StringIndex;
-    /// 
-    /// let s = String::from("Hello");
-    /// assert_eq!('H', s.byte_at(0) as char);
-    /// assert_eq!('e', s.byte_at(1) as char);
-    /// assert_eq!('l', s.byte_at(2) as char);
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::StringIndex;
-    /// 
-    /// let s = "Hello";
-    /// assert_eq!('H', s.byte_at(0) as char);
-    /// assert_eq!('e', s.byte_at(1) as char);
-    /// assert_eq!('l', s.byte_at(2) as char);
-    /// ```
-    #[must_use]
-    fn byte_at(&self, index: usize) -> u8;
-}
-/// Trait for functions related to brackets.
-pub trait Brackets {
-    /// Determines whether the given `&str` or `String` has valid brackets.
-    /// # Returns
-    /// `Ok(true)` if all brackets were closed, otherwise `Err(usize)`, where `usize` is the index of the String at which the error occured.
-    /// # Examples
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = "([{Some text won't throw it off}])";
-    /// 
-    /// assert_eq!(Ok(true), s.validate_brackets());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = String::from("([{}])");
-    /// 
-    /// assert_eq!(Ok(true), s.validate_brackets());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = "([{}]";
-    /// 
-    /// assert_eq!(Err(5), s.validate_brackets());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = "([{Some text won't throw it off}]))";
-    /// 
-    /// assert_eq!(Err(34), s.validate_brackets());
-    /// ```
-    /// ```
-    /// use lib_rapid::compsci::general::Brackets;
-    /// 
-    /// let s = "([{]}))";
-    /// 
-    /// assert_eq!(Err(3), s.validate_brackets());
-    /// ```
-    fn validate_brackets(&self) -> Result<bool, usize>;
-}
 /// Simple methods for characters.
 pub trait CharTools {
     /// Determines whether a character is a opening bracket.
@@ -371,8 +313,7 @@ pub trait FloatMagic {
     /// # Panics
     /// Panics if one of the parts is out of range for `Self`.
     /// # Examples
-    /// ```    fn raw_exponent(&self) -> Self::Exponent {
-
+    /// ```
     /// use lib_rapid::compsci::general::FloatMagic;
     /// 
     /// let my_float: f32 = 3.1415927;
@@ -380,13 +321,13 @@ pub trait FloatMagic {
     /// let exponent      = my_float.raw_exponent();
     /// let mantissa      = my_float.raw_mantissa();
     /// 
-    /// let my_double       = my_float as f64;
-    /// let sign_1          = my_double.is_sign_negative() as u8;
-    /// let exponent_1      = my_double.raw_exponent();
-    /// let mantissa_1      = my_double.raw_mantissa();
+    /// let my_double: f64 = 3.141592653589793;
+    /// let sign_1         = my_double.is_sign_negative() as u8;
+    /// let exponent_1     = my_double.raw_exponent();
+    /// let mantissa_1     = my_double.raw_mantissa();
     /// 
     /// assert_eq!(3.1415927, f32::raw_compose(sign, exponent, mantissa));
-    /// assert_eq!(3.1415927410125732, f64::raw_compose(sign, exponent_1, mantissa_1));
+    /// assert_eq!(3.141592653589793, f64::raw_compose(sign_1, exponent_1, mantissa_1));
     /// ```
     #[must_use]
     fn raw_compose(sign: u8, exponent: Self::Exponent, mantissa: Self::Mantissa) -> Self;
@@ -395,19 +336,15 @@ pub trait FloatMagic {
 impl CharTools for char {
     fn is_open_bracket(&self) -> bool {
         match self {
-            '(' => { true }
-            '{' => { true }
-            '[' => { true }
-            _   => { false }
+            '(' | '{' | '[' => { true }
+            _               => { false }
         }
     }
 
     fn is_closed_bracket(&self) -> bool {
         match self {
-            ')' => { true }
-            '}' => { true }
-            ']' => { true }
-            _   => { false }
+            ')' | '}' | ']' => { true }
+            _               => { false }
         }
     }
 
@@ -476,8 +413,8 @@ impl FloatMagic for f32 {
 }
 
 impl FloatMagic for f64 {
-    type Mantissa = u64;
-    type Exponent = u16;
+    type Mantissa     = u64;
+    type Exponent     = u16;
     type RealExponent = i32;
 
     #[inline]
@@ -513,92 +450,11 @@ impl FloatMagic for f64 {
     fn raw_compose(sign: u8, exponent: Self::Exponent, mantissa: Self::Mantissa) -> Self {
         if sign     > 1                { panic!("A sign bigger than 1 is not allowed."); }
         if exponent > 2047             { panic!("An exponent bigger than 2047 is not allowed."); }
-        if mantissa > 4503599627370000 { panic!("A mantissa bigger than 8388607 is not allowed."); }
+        if mantissa > 4503599627370000 { panic!("A mantissa bigger than 4503599627370000 is not allowed."); }
 
         unsafe { transmute((mantissa |
-                          ((exponent as u64) << 23)) |
-                           (sign     as u64) << 31) }
-    }
-}
-
-impl Brackets for &str {
-    fn validate_brackets(&self) -> Result<bool, usize> {
-        let mut s: Vec<char> = Vec::with_capacity(self.len());
-        let mut i: usize = 0;
-        for c in self.chars() {
-            match c {
-                '[' => { s.push(c); },
-                '{' => { s.push(c); },
-                '(' => { s.push(c); },
-                ']' => { 
-                         if s.pop() != Some('[')
-                         { return Err(i); }
-                        },
-                '}' => { if s.pop() != Some('{')
-                         { return Err(i); }
-                        },
-                ')' => { if s.pop() != Some('(')
-                         { return Err(i); }
-                        },
-                _   => { }
-            }
-            i.inc();
-        }
-
-        if !s.is_empty()
-        { return Err(i); }
-        Ok(true)
-    }
-}
-
-impl Brackets for String {
-    fn validate_brackets(&self) -> Result<bool, usize> {
-        let mut s: Vec<char> = Vec::with_capacity(self.len());
-        let mut i: usize = 0;
-        for c in self.chars() {
-            match c {
-                '[' => { s.push(c); },
-                '{' => { s.push(c); },
-                '(' => { s.push(c); },
-                ']' => { if s.pop() != Some('[')
-                         { return Err(i); }
-                        },
-                '}' => { if s.pop() != Some('{')
-                         { return Err(i); }
-                        },
-                ')' => { if s.pop() != Some('(')
-                         { return Err(i); }
-                        },
-                _   => { }
-            }
-            i.inc();
-        }
-
-        if !s.is_empty()
-        { return Err(i); }
-        Ok(true)
-    }
-}
-
-impl StringIndex for String {
-    #[inline]
-    fn char_at(&self, index: usize) -> Option<char> {
-        self.chars().nth(index)
-    }
-    #[inline]
-    fn byte_at(&self, index: usize) -> u8 {
-        self.as_bytes()[index]
-    }
-}
-
-impl StringIndex for &str {
-    #[inline]
-    fn char_at(&self, index: usize) -> Option<char> {
-        self.chars().nth(index)
-    }
-    #[inline]
-    fn byte_at(&self, index: usize) -> u8 {
-        self.as_bytes()[index]
+                          ((exponent as u64) << 52)) |
+                           (sign     as u64) << 63) }
     }
 }
 
@@ -635,16 +491,8 @@ impl<T: BitXorAssign +
     where
     <T as TryFrom<U>>::Error: Debug {
     fn xor_with(&self, other: &[U]) -> Vec<T> {
-        if size_of_val(self) != size_of_val(other)
-        { panic!("{}", BITWISE_ERR1); }
-
-        let t_size: usize = size_of::<T>();
-        let u_size: usize = size_of::<U>();
-        if t_size < u_size
-        { panic!("{}", BITWISE_ERR2); }
-        
+        let multiplier: usize  = bitwise_preproc::<T, U>(self, other);
         let mut _res:   Vec<T> = self.to_vec();
-        let multiplier: usize  = t_size / u_size;
 
         for (index, slice) in other.chunks(multiplier).enumerate() {
             _res[index] ^= bitwise_from(slice);
@@ -654,16 +502,9 @@ impl<T: BitXorAssign +
     }
 
     fn or_with(&self, other: &[U]) -> Vec<T> {
-        if size_of_val(self) != size_of_val(other)
-        { panic!("{}", BITWISE_ERR1); }
-
-        let t_size: usize = size_of::<T>();
-        let u_size: usize = size_of::<U>();
-        if t_size < u_size
-        { panic!("{}", BITWISE_ERR2); }
-
+        let multiplier: usize  = bitwise_preproc::<T, U>(self, other);
         let mut _res:   Vec<T> = self.to_vec();
-        let multiplier: usize  = t_size / u_size;
+
         for (index, slice) in other.chunks(multiplier).enumerate() {
             _res[index] |= bitwise_from(slice);
         }
@@ -672,16 +513,8 @@ impl<T: BitXorAssign +
     }
 
     fn and_with(&self, other: &[U]) -> Vec<T> {
-        if size_of_val(self) != size_of_val(other)
-        { panic!("{}", BITWISE_ERR1); }
-
-        let t_size: usize = size_of::<T>();
-        let u_size: usize = size_of::<U>();
-        if t_size < u_size
-        { panic!("{}", BITWISE_ERR2); }
-        
+        let multiplier: usize  = bitwise_preproc::<T, U>(self, other);
         let mut _res:   Vec<T> = self.to_vec();
-        let multiplier: usize  = t_size / u_size;
 
         for (index, slice) in other.chunks(multiplier).enumerate() {
             _res[index] &= bitwise_from(slice);
@@ -691,6 +524,40 @@ impl<T: BitXorAssign +
     }
 }
 
+impl<T: Clone + std::cmp::PartialEq> SliceOps<T> for [T] {
+    #[inline]
+    fn peek(&self) -> T {
+        self[self.len() - 1].clone()
+    }
+    #[inline]
+    fn swap_top(&mut self) {
+        let a: T = self[self.len() - 1].clone();
+        let b: T = self[self.len() - 2].clone();
+
+        self[self.len() - 1] = b;
+        self[self.len() - 2] = a;
+    }
+
+    fn count_of(&self, item: T) -> usize {
+        let mut res: usize = 0;
+        for i in self {
+            if i == &item
+            { res.inc(); }
+        }
+        res
+    }
+
+    fn positions_of(&self, item: T) -> Vec<usize> {
+        let mut res = Vec::with_capacity(self.len());
+        for i in self.iter().enumerate() {
+            if i.1 == &item
+            { res.push(i.0); }
+        }
+        res
+    }
+}
+
+#[must_use]
 fn bitwise_from<T: From<u8> +
                    From<U> +
                    Shl<Output = T> +
@@ -700,8 +567,20 @@ fn bitwise_from<T: From<u8> +
     let mut end_prod: T = T::from(0);
 
     for (inner_idx, inner_ref) in slice.iter().rev().enumerate() {
-        end_prod |= T::from(*inner_ref) << (inner_idx as u8 * 8).into();
+        end_prod |= T::from(*inner_ref) << ((inner_idx as u8) << 3u8).into();
     }
 
     end_prod
+}
+
+#[inline(always)]
+fn bitwise_preproc<T,U>(a: &[T], b: &[U]) -> usize {
+    if size_of_val(a) != size_of_val(b)
+    { panic!("{}", BITWISE_ERR1); }
+
+    let t_size: usize = size_of::<T>();
+    let u_size: usize = size_of::<U>();
+    if t_size < u_size
+    { panic!("{}", BITWISE_ERR2); }
+    t_size / u_size
 }
\ No newline at end of file
